//********************************************
//* (c) 2015 Cristian Copcea                 *
//* Please retain here the credits           *
//* just add yours if you make modifications *
//* to this file                             *
//********************************************
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#define F_CPU 12000000L
#include <util/delay.h>


//#include "i2cmaster/i2cmaster.h"
#include "ds1621.h"
#include "ds18b20.h"
#include "rtc.h"


//keys definition
#define KEYSELECT 	3
#define KEYSET 		2
#define KPIN  		PIND


//segment definitions
#define SEG_NULL	0x00
#define SEG_a 		0x01
#define SEG_b 		0x02
#define SEG_c 		0x04
#define SEG_d 		0x08
#define SEG_e 		0x10
#define SEG_f 		0x20
#define SEG_g 		0x40
#define SEG_dot		0x80

//clock working mode definitions
#define	SHOWCLOCK		0x01
#define SHOWTEMP		0x02
#define SHOWDATE		0x03
#define SHOWYEAR		0x04
#define	SETHOURS		0x05
#define SETMINUTES		0x06
#define SETDATE			0x07
#define SETMONTH		0x08
#define SETYEAR			0x09
#define	SETALHOURS		0x0A
#define SETALMINUTES	0x0B
#define SETAL			0x0C
#define	SETSECMODE		0x0D
#define	SHOWSENSORS		0x0E

//seconds display working mode
uint8_t SecMode=0;
uint8_t MinSecMode=1;
uint8_t MaxSecMode=9;

//Alarm related variables
uint8_t ALMinutes=0;
uint8_t ALHours=0;
uint8_t ALSet=0;
uint8_t AlarmOn=0;

//------------------------------------------------------------------------------
// EEMEM variables making the clock perform otherwise

//ECFUnit defines Celsius or Fahrenheit display
//ECFUnit=0 means Celsius Display
//ECFUnit=1 defines Fahnrenheit Display
EEMEM uint8_t ECFUnit=0;
uint8_t CFUnit;

//This variable makes the clock show alternatively the hour and the temperature
//ESwing=0 means normal Display
//ESwing=1 means alternative Clock and Temperature Display
EEMEM uint8_t ESwing=1;
uint8_t Swing;

//This variable makes the clock show the hour in 12H or 24H mode
//EUSMode=0 means 24H display
//EUSMode=1 means 12H display
EEMEM uint8_t EUSMode=0;
uint8_t USMode;

//EEPROM variables
EEMEM uint8_t ESecMode=9;
EEMEM uint8_t EALMinutes=0;
EEMEM uint8_t EALHours=0;
EEMEM uint8_t EALSet=0;
//ECRC and DefCRC have to be recomputed manually after a modification to any of the following eeprom variables:
//ECFUnit, ESwing, EUSMode, ESecMode, EALMinutes, EALHours, EALSet
//ECRC=ECFUnit+ESwing+EUSMode+ESecMode+EALMinutes+EALHours+EALSet
EEMEM uint8_t ECRC=10;
uint8_t DefCRC=10;


//Default values vor EEPROM variables if CRC is not correct.
//Alarm default hour values should be set in the interval 0-23 even for US Mode
uint8_t DefCFUnit=0;
uint8_t DefSwing=1;
uint8_t DefUSMode=0;
uint8_t DefSecMode=9;
uint8_t DefALMinutes=0;
uint8_t DefALHours=0;
uint8_t DefALSet=0;

uint8_t CRC;


//---------------------------------------------------------------------------------

//variables minimums and maximums definitions
#define MinALMinutes	0
#define MinALHours		0
#define MinALSet		0
#define MaxALMinutes	59
#define MaxALHours		23
#define MaxALSet		1
#define MinYears		10 //(i.e.2010)
#define MinMonth		1
#define MinDate			1
#define MinHours		0
#define MinMinutes		0
#define MaxYears		100
#define MaxMonth		12
#define MaxDate			31
#define MaxHours		23
#define MaxMinutes		59




//global variables 
uint16_t digit, decimal, digit_addressed=0;
uint8_t seconds;
uint8_t d[18];
dateTime dt,dt1;
uint8_t Start=1;
uint8_t temperature;
uint8_t timestamp1,timestamp2;
uint16_t refresh=0;
uint8_t refresh_resetted =0;
uint8_t pulsing=0, showdigit=0;

//clock working mode
uint8_t ClockMode=SHOWCLOCK;

//variables used for various delays
uint16_t t1=0, t2=0, t3=0;

//variables used for 18X20 sensors
#define MAXSENSORS 5
uint8_t nSensors;
int8_t gSensorIDs[MAXSENSORS][DS18B20_ROMCODE_SIZE];

//TEMPDISPLAY works like this:
//if it is even, it shows the sensor number /2 +1
//if it is odd, it shows the temperature of the sensor number/2
//pressing the key will increment the variable
//leaving the key not pressed reverts the variable back to zero

uint8_t TEMPDISPLAY=0;

//7-segments character definitions
unsigned char seg[]=
{
	(SEG_a|SEG_b|SEG_c|SEG_d|SEG_e|SEG_f), 			// 0
	(SEG_b|SEG_c), 									// 1
	(SEG_a|SEG_b|SEG_d|SEG_e|SEG_g), 				// 2
	(SEG_a|SEG_b|SEG_c|SEG_d|SEG_g), 				// 3
	(SEG_b|SEG_c|SEG_c|SEG_f|SEG_g), 				// 4
	(SEG_a|SEG_c|SEG_d|SEG_f|SEG_g), 				// 5
	(SEG_a|SEG_c|SEG_d|SEG_e|SEG_f|SEG_g), 			// 6
	(SEG_a|SEG_b|SEG_c), 							// 7
	(SEG_a|SEG_b|SEG_c|SEG_d|SEG_e|SEG_f|SEG_g), 	// 8
	(SEG_a|SEG_b|SEG_c|SEG_d|SEG_f|SEG_g), 			// 9
	(SEG_dot), 										// 10 NULL DISPLAY (decimal point lit)
	(SEG_a|SEG_b|SEG_f|SEG_g), 						// 11 THE SYMBOL "o" (degrees)
	(SEG_a|SEG_d|SEG_e|SEG_f), 						// 12 THE SYMBOL "C"
	(SEG_c|SEG_e|SEG_g),	 						// 13 THE SYMBOL "n"
	(SEG_a|SEG_e|SEG_f|SEG_g), 						// 14 THE SYMBOL "F"
	(SEG_c|SEG_d|SEG_e|SEG_g), 						// 15 THE SYMBOL "o" (like small o)
	(SEG_NULL), 									// 16 NULL SYMBOL (no decimal point)
	(SEG_a|SEG_b|SEG_c|SEG_e|SEG_f|SEG_g),			// 17 THE SYMBOL "A"
	(SEG_b|SEG_c|SEG_d|SEG_e|SEG_g),				// 18 THE SYMBOL "d"
	(SEG_d|SEG_e|SEG_f|SEG_g),						// 19 THE SYMBOL "t"
	(SEG_d|SEG_e|SEG_f),							// 20 THE SYMBOL "L"
	(SEG_a|SEG_d|SEG_e|SEG_f|SEG_g),				// 21 THE SYMBOL "E"
	(SEG_a|SEG_b|SEG_e|SEG_f|SEG_g)					// 22 THE SYMBOL "P"
};


//interrupt routine for character and seconds display into an unending loop.
//it uses the second 16-bit timer, TIMER1
ISR(TIMER1_OVF_vect)
{
void SetPortA (uint8_t i)
{
		if (showdigit) PORTA = ~(d[i]); else PORTA= ~(SEG_NULL);
		PORTC=0xFF;
};
void SetPortACD(uint8_t i, uint8_t j)
{
		PORTD= 0xF0; //set nil to digits
		PORTA=0xFF;  //Led data is nil
		PORTC=0xFF;  //deselect all led rows
		PORTC=j;	//Select Bit0 - first 8 leds
		_delay_us(10);
		PORTA = ~(d[i]); //set led data to display
};

//	uint8_t PV;
//comment up to the next sign in order to use first version of function for seconds #9
//	|
//	|
//	v

	refresh++;
	timestamp1=dt.second;
	if (timestamp1==timestamp2)
	{
		if (!refresh_resetted)
		{
			refresh=0;
			refresh_resetted=1;
		}
	}
	else
	{
		refresh_resetted=0;
	}

//	^
//	|
//	|
//comment from the first sign in order to use first version of function for seconds #9
	if (pulsing) showdigit=(refresh<512); else showdigit=1;
	switch (digit_addressed)
	{
		case 0: 
		SetPortA(0);
		//Set bit 4, clear bits 5 to 7)
		PORTD=(PORTD & ~(1<<4)) | (1<<5) | (1<<6) | (1<<7);
		break;
		case 1: 
		SetPortA(1);
		PORTD=(PORTD & ~(1<<5)) | (1<<4) | (1<<6) | (1<<7);
		break;
		case 2: 
		SetPortA(2);
		PORTD=(PORTD & ~(1<<6)) | (1<<4) | (1<<5) | (1<<7);
		break;
		case 3: 
		SetPortA(3);
		PORTD=(PORTD & ~(1<<7)) | (1<<4) | (1<<5) | (1<<6);
		break;
		case 4: 
		SetPortACD(10,0xFE);
		break;
		case 5: 
		SetPortACD(11,0xFD);
		break;
		case 6: 
		SetPortACD(12,0xFB);
		break;
		case 7: 
		SetPortACD(13,0xF7);
		break;
		case 8: 
		SetPortACD(14,0xEF);
		break;
		case 9: 
		SetPortACD(15,0xDF);
		break;
		case 10: 
		SetPortACD(16,0xBF);
		break;
		case 11: 
		SetPortACD(17,0x7F);
		break;
	}
	digit_addressed++;
	if(digit_addressed>=12) digit_addressed=0;
//comment the following line in order to use first version of function for seconds #9
//	|
//	|
//	v

	timestamp2=dt.second;

//	^
//	|
//	|
}


uint8_t crc8(void)
{
	return CFUnit+Swing+USMode+SecMode+ALMinutes+ALHours+ALSet;
}


void crc_write(void)
{

	uint8_t a,b,c,d,e,f,g;
	a= eeprom_read_byte(&ECFUnit);
	b= eeprom_read_byte(&ESwing);
	c= eeprom_read_byte(&EUSMode);
	d= eeprom_read_byte(&ESecMode);
	e= eeprom_read_byte(&EALMinutes); 
	f= eeprom_read_byte(&EALHours);
	g= eeprom_read_byte(&EALSet);
	eeprom_write_byte(&ECRC,a+b+c+d+e+f+g);
}


//tool function to calculate the power of 2
uint8_t p2(uint8_t exponent)
{
	uint8_t a=1,b=1,i;
	for(i=0;i<=exponent-1;i++)
	{
		b*=2;
		a+=b;
	}
	return a;
}

//tool function to convert from decimal to binary display
uint8_t dectobin(uint8_t number)
{
	uint8_t a,i;
	a=1;
	if (number==0) 	a=1;
	else
		for (i=0;i<number;i++) a=a*2;
	return a;
}

//tool function to show if an integer is odd or even
uint8_t odd(uint8_t num)
{
	if (num-num/2*2 !=0)
	return 1;
	else
	return 0;
}



uint8_t bb(uint8_t in11, uint8_t sec1, uint8_t sec2) //function to reduce the code
{
	return dectobin(seconds-in11*8)*((seconds>= sec1) && (seconds<= sec2));
}

void cc(uint8_t d0,uint8_t d1,uint8_t d2,uint8_t d3,uint8_t d4,uint8_t d5,uint8_t d6,uint8_t d7, uint8_t in111)
{

			d[10]=d0 | bb(in111,0,7);
			d[11]=d1 | bb(in111,8,15);
			d[12]=d2 | bb(in111,16,23);
			d[13]=d3 | bb(in111,24,31);
			d[14]=d4 | bb(in111,32,39);
			d[15]=d5 | bb(in111,40,47);
			d[16]=d6 | bb(in111,48,55);
			d[17]=d7 | bb(in111,56,59);
}

void fillbefore(uint8_t in11,uint8_t value)
{
	uint8_t i;
	for (i=0;i<in11;i++)
	{
		d[i+10]=value;
	}

}

void fillafter(uint8_t in11,uint8_t value)
{
	uint8_t i;
	for (i=in11+11;i<18;i++)
	{
		d[i]=value;
	}
}

//main display function
void display(void)
{

	uint8_t hr, min, month, date, temp,in1,i,j;
	uint16_t year;
	uint8_t temphr,tempmin;

	void SetD(uint8_t d0,uint8_t d1,uint8_t d2,uint8_t d3)
	{
	d[0]=d0;
	d[1]=d1;
	d[2]=d2;
	d[3]=d3;
	}

	void SetFill(uint8_t in11,uint8_t fb2, uint8_t dd2, uint8_t fa2)
	{
			fillbefore(in11,fb2);
			d[in11+10]=dd2;
			fillafter(in11,fa2);
	}

	void DisplayHours(uint8_t i, uint8_t j)
	{
		if (USMode)
		{
			if (i>12) i-=12;
			if (i==0) i=12;
		}
		d[4]= i/10;
		d[5]= i-d[4]*10;
		if (USMode)
			{
				if (j>12) 
					SetD(seg[d[4]],seg[d[5]],seg[16],seg[22]);
				else
					SetD(seg[d[4]],seg[d[5]],seg[16],seg[17]);
			}
		else
			SetD(seg[d[4]],seg[d[5]]+SEG_dot,seg[16],seg[16]);
	}

	if (ClockMode==SHOWSENSORS)
	{
		SetD(seg[19],seg[21]+SEG_dot,SEG_NULL,seg[digit]);
		//d[0]= seg[19];
		//d[1]= seg[21]+SEG_dot;
		//d[2]= SEG_NULL;
		//d[3]= seg[digit];
	}
	else
	if (ClockMode==SHOWCLOCK)
	{
		//digit=refresh/13;
		if (t2==0)//show clock
		{
		if (USMode)
		{
			tempmin=digit%100;
			temphr=digit/100;
			if (temphr>12) temphr-=12;
			if (temphr==0) temphr=12;
			digit=tempmin+100*temphr;
		}

		//computing hours and minutes
		hr=digit/100;
		min=digit-hr*100;
		d[4]= hr/10; 
		d[5]= hr-d[4]*10;
		d[6]= min/10;
		d[7]= min-d[6]*10;
		if (hr>9) d[0]= seg[d[4]]; else d[0]= SEG_NULL;
		d[1]= seg[d[5]];
		//add dot point on odd seconds
		if (refresh<512) d[1]+=SEG_dot;
		d[2]= seg[d[6]];
		d[3]= seg[d[7]];
		}
		else
		if (t2<5) //show "date" while setting the clock
		{
		SetD(seg[18],seg[17],seg[19],seg[21]);
		//d[0]= seg[18];
		//d[1]= seg[17];
		//d[2]= seg[19];
		//d[3]= seg[21];
		}
		else
		if (t2<=10) //show "S :AL" (Set Alarm) while setting the clock
		{
		SetD(seg[5],seg[10],seg[17],seg[20]);
		//d[0]= seg[5];
		//d[1]= seg[10];
		//d[2]= seg[17];
		//d[3]= seg[20];
		}
		else		//show "S :CL" (Set clock) while setting the clock
		{
		SetD(seg[5],seg[10],seg[12],seg[20]);
		//d[0]= seg[5];
		//d[1]= seg[10];
		//d[2]= seg[12];
		//d[3]= seg[20];
		}
	}
	else
	if (ClockMode==SHOWDATE)
	{
		//computing hours and minutes
		month=digit/100;
		date=digit-month*100;
		d[4]= month/10;
		d[5]= month-d[4]*10;
		d[6]= date/10;
		d[7]= date-d[6]*10;
		if (d[4]!=0) 	d[0]= seg[d[4]];
		else			d[0]= seg[16];
		//d[0]= seg[d[4]];
		d[1]= seg[d[5]];
		if (d[6]!=0) 	d[2]= seg[d[6]];
		else			d[2]= seg[16];
		//d[2]= seg[d[6]];
		d[3]= seg[d[7]];
	}
	if (ClockMode==SHOWYEAR)
	{
		//computing hours and minutes
		year=digit;
		d[4]= year/1000;
		year-=d[4]*1000;
		d[5]= year/100;
		year-=d[5]*100;
		d[6]= year/10;
		year-=d[6]*10;
		SetD(seg[d[4]],seg[d[5]],seg[d[6]],seg[d[7]]);
		d[7]= year;
		//d[0]= seg[d[4]];
		//d[1]= seg[d[5]];
		//d[2]= seg[d[6]];
		//d[3]= seg[d[7]];
	}
	else
	if (ClockMode==SHOWTEMP)
	{
		if (nSensors==0)
		{
			SetD(SEG_NULL,seg[13],seg[15],SEG_NULL);
			//d[0]= SEG_NULL;
			//d[1]= seg[13];
			//d[2]= seg[15];
			//d[3]= SEG_NULL;
		}
		else
		if (!odd(TEMPDISPLAY))
		{
		//showing the sensor number /2
			SetD(seg[19],seg[21]+SEG_dot,SEG_NULL,seg[TEMPDISPLAY/2+1]);
			//d[0]= seg[19];
			//d[1]= seg[21]+SEG_dot;
			//d[2]= SEG_NULL;
			//d[3]= seg[TEMPDISPLAY/2+1];
		}
		else
		{
		//displaying the temperature of the sensor number/2 -1
			temp=digit;
			d[4]= temp/100;
			temp-=d[4]*100;
			d[5]= temp/10;
			temp-=d[5]*10;
			d[6]= temp;
			if (d[4]==0) d[0]=SEG_NULL; else d[0]= seg[d[4]];
			if ((d[5]==0) && (d[4]==0)) d[1]=SEG_NULL; else d[1]= seg[d[5]];
			d[2]= seg[d[6]];
			d[3]= seg[12+CFUnit*2];
		}
	}
	else
	if (ClockMode==SETHOURS)
	{
		temphr=dt1.hour;
		DisplayHours(temphr,dt1.hour);
	}
	else
	if (ClockMode==SETMINUTES)
	{
		d[6]= dt1.minute/10;
		d[7]= dt1.minute-d[6]*10;
		SetD(seg[16],seg[16]+SEG_dot,seg[d[6]],seg[d[7]]);
		//d[0]= seg[16];
		//d[1]= seg[16]+SEG_dot;
		//d[2]= seg[d[6]];
		//d[3]= seg[d[7]];
	}
	else
	if (ClockMode==SETDATE)
	{
		d[6]= dt1.date/10;
		d[7]= dt1.date-d[6]*10;
		SetD(seg[16],seg[16],seg[d[6]],seg[d[7]]);
		//d[0]= seg[16];
		//d[1]= seg[16];
		//d[2]= seg[d[6]];
		//d[3]= seg[d[7]];
	}
	else
	if (ClockMode==SETMONTH)
	{
		d[4]= dt1.month/10;
		d[5]= dt1.month-d[4]*10;
		SetD(seg[d[4]],seg[d[5]],seg[16],seg[16]);
		//d[0]= seg[d[4]];
		//d[1]= seg[d[5]];
		//d[2]= seg[16];
		//d[3]= seg[16];
	}
	else
	if (ClockMode==SETYEAR)
	{
		d[4]= 2;
		d[5]= dt1.year/100;
		d[6]= (dt1.year-d[5]*100)/10;
		d[7]= dt1.year-d[5]*100-d[6]*10;
		SetD(seg[d[4]],seg[d[5]],seg[d[6]],seg[d[7]]);
		//d[0]= seg[d[4]];
		//d[1]= seg[d[5]];
		//d[2]= seg[d[6]];
		//d[3]= seg[d[7]];
	}
	else
	if (ClockMode==SETALHOURS)
	{
		d[4]= ALHours/10;
		d[5]= ALHours-d[4]*10;
		SetD(seg[d[4]],seg[d[5]]+SEG_dot,seg[16],seg[16]);
		//d[0]= seg[d[4]];
		//d[1]= seg[d[5]]+SEG_dot;
		//d[2]= seg[16];
		//d[3]= seg[16];
		temphr=ALHours;
		DisplayHours(ALHours,ALHours);

	}
	else
	if (ClockMode==SETALMINUTES)
	{
		d[6]= ALMinutes/10;
		d[7]= ALMinutes-d[6]*10;
		SetD(seg[16],seg[16]+SEG_dot,seg[d[6]],seg[d[7]]);
		//d[0]= seg[16];
		//d[1]= seg[16]+SEG_dot;
		//d[2]= seg[d[6]];
		//d[3]= seg[d[7]];
	}
	else
	if (ClockMode==SETAL)
	{
		if (ALSet==1) //on
		{
		SetD(seg[15],seg[13],seg[16],seg[16]);
			//d[0]= seg[15];
			//d[1]= seg[13];
			//d[2]= seg[16];
			//d[3]= seg[16];
		}
		else	//off
		{
		SetD(seg[15],seg[14],seg[14],seg[16]);
			//d[0]= seg[15];
			//d[1]= seg[14];
			//d[2]= seg[14];
			//d[3]= seg[16];
		}
	}
	else
	if (ClockMode==SETSECMODE)
	{
		d[6]= SecMode;
		SetD(seg[16],seg[16],seg[16],seg[d[6]]);
		//d[0]= seg[16];
		//d[1]= seg[16];
		//d[2]= seg[16];
		//d[3]= seg[d[6]];
	}

	//computing seconds -> multiple choices available pertaining to the design we wish
	switch(SecMode)
	{
		case 1: //seconds go normally
			in1=seconds/8;
			SetFill(in1,0x00,dectobin(seconds-in1*8),0x00);
			//fillbefore(in1,0x00);
			//d[in1+10]=dectobin(seconds-in1*8);
			//fillafter(in1,0x00);
			break;
		case 2: //seconds are all lit except the current one
			in1=seconds/8;
			SetFill(in1,0xFF,~dectobin(seconds-in1*8),0xFF);
			//fillbefore(in1,0xFF);
			//d[in1+10]=~dectobin(seconds-in1*8);
			//fillafter(in1,0xFF);
			break;
		case 3: //seconds fill progressively
			in1=seconds/8;
			for (i=0;i<in1;i++)
			SetFill(in1,0xFF,p2(seconds-in1*8),0x00);
			//fillbefore(in1,0xFF);
			//d[in1+10]=p2(seconds-in1*8);
			//fillafter(in1,0x00);
			break;
		case 4:	//seconds empty progressively
			in1=seconds/8;
			SetFill(in1,0x00,~p2(seconds-in1*8),0xFF);
			//fillbefore(in1,0x00);
			//d[in1+10]=~p2(seconds-in1*8);
			//fillafter(in1,0xFF);
			break;
		case 5://seconds fill progressively at odd minutes and empty progressively at even minutes
			if (odd(dt.minute))
			{
				in1=seconds/8;
				SetFill(in1,0xFF,p2(seconds-in1*8),0x00);
				//fillbefore(in1,0xFF);
				//d[in1+10]=p2(seconds-in1*8);
				//fillafter(in1,0x00);
			}
			else
			{
				in1=seconds/8;
				SetFill(in1,0x00,~p2(seconds-in1*8),0xFF);
				//fillbefore(in1,0x00);
				//d[in1+10]=~p2(seconds-in1*8);
				//fillafter(in1,0xFF);
			}
			break;
		case 6: //seconds go normally and red leds lit
			in1=seconds/8;
			cc(0b00100001,0b10000100,0b00010000,0b01000010,0b00001000,0b00100001,0b10000100,0b00010000,in1);
			break;
		case 7:	//seconds go normally and red leds lit. Red leds shift when the current second falls on them
			in1=seconds/8;
			if (seconds-seconds/5*5!=0)
				cc(0b00100001,0b10000100,0b00010000,0b01000010,0b00001000,0b00100001,0b10000100,0b00010000,in1);
			else
			if (odd(seconds/5))
				cc(0b01000010,0b00001000,0b00100001,0b10000100,0b00010000,0b01000010,0b00001000,0b00100001,in1);
			else
				cc(0b00010000,0b01000010,0b00001000,0b00100001,0b10000100,0b00010000,0b01000010,0b00001000,in1);
			break;
		case 8:	//seconds go normally and red leds lit. Red leds go off when the current second falls on them
			in1=seconds/8;
			if (seconds-seconds/5*5!=0)
				cc(0b00100001,0b10000100,0b00010000,0b01000010,0b00001000,0b00100001,0b10000100,0b00010000,in1);
			else
				cc(0,0,0,0,0,0,0,0,in1);
			break;
		case 9:	
			j=refresh/18; // /13 for 8MHz, /19 for 12MHz
			if (odd(dt.minute))
			{
				if (j<=seconds)
				{
					in1=j/8;
					SetFill(in1,0xFF,p2(j-in1*8),0x00);
				}
			}
			else
			{
				if (j<=seconds)
				{
					in1=j/8;
					SetFill(in1,0x00,~p2(j-in1*8),0xFF);
				}
			}
			break;
		default:
			break;
	}
}

//function that beeps
void beep(void)
{
	PORTB &= ~_BV(4);
	_delay_ms(100);
	PORTB |= _BV(4);
}


uint8_t Search_sensors(void)
{
	uint8_t i;
	uint8_t id[DS18B20_ROMCODE_SIZE];
	uint8_t diff, nSensors;
	
	ds18b20_reset();

	nSensors = 0;
	
	diff = DS18B20_SEARCH_FIRST;
	while ( diff != DS18B20_LAST_DEVICE && nSensors < MAXSENSORS ) {
		DS18X20_find_sensor( &diff, &id[0] );
		
		if( diff == DS18B20_PRESENCE_ERR ) {
			//uart_puts_P( "No Sensor found" NEWLINESTR );
			break;
		}
		
		if( diff == DS18B20_DATA_ERR ) {
			//uart_puts_P( "Bus Error" NEWLINESTR );
			break;
		}
		
		for ( i=0; i < DS18B20_ROMCODE_SIZE; i++ )
			gSensorIDs[nSensors][i] = id[i];
		
		nSensors++;
	}
	
	return nSensors;
}

void init(void)
{
	//Timer 0 stopped
	TCNT0 = 0xFF ;
	TCCR0 = 0x00; 
	//Timer 1 enabled
	TCNT1 = 0xFF ;
	TCCR1A = 0x02; 	//9 bit PWM with external pins disconnected
	TCCR1B = 0x02; 	//prescaler of 8
	TIMSK = 0x82; 	//timer overflow interrupts enabled
	GICR = 0x00; 	//external interrupts disabled: will manage switch pushes in software
	sei();			//enable interrupts

//PORTS
	PORTA=0xFF;		
	DDRA =0xFF;		
	DDRB |=_BV(0) | _BV(4);
	PORTB|=_BV(0) | _BV(1) | _BV(2) | _BV(3)| _BV(4);
	PORTC=0xFF;
	DDRC =0xFF;
	
	PORTD=0xff;
	DDRD =_BV(0) | _BV(1) |_BV(4)|_BV(5)|_BV(6)|_BV(7);
	timestamp1=timestamp2=dt.second;
	//timestamp2=dt.second;

	//read EEprom values to current variables
	CFUnit= eeprom_read_byte(&ECFUnit);
	Swing= eeprom_read_byte(&ESwing);
	USMode= eeprom_read_byte(&EUSMode);
	SecMode= eeprom_read_byte(&ESecMode);
	ALMinutes= eeprom_read_byte(&EALMinutes); 
	ALHours= eeprom_read_byte(&EALHours);
	ALSet= eeprom_read_byte(&EALSet);
	CRC= eeprom_read_byte(&ECRC);
	if (CRC!=crc8()) 
	{
	//CRC in EEPROM incorrect => 
		//replace all values with default values 
		CFUnit=DefCFUnit;
		Swing=DefSwing;
		USMode=DefUSMode;
		SecMode=DefSecMode;
		ALMinutes=DefALMinutes;
		ALHours=DefALHours;
		ALSet=DefALSet;	
		//rewrite default values in EEPROM
		eeprom_write_byte(&ECFUnit,CFUnit);
		eeprom_write_byte(&ESwing,Swing);
		eeprom_write_byte(&EUSMode,USMode);
		eeprom_write_byte(&ESecMode,SecMode);
		eeprom_write_byte(&EALMinutes,ALMinutes);
		eeprom_write_byte(&EALHours,ALHours);
		eeprom_write_byte(&EALSet,ALSet);
		eeprom_write_byte(&ECRC,DefCRC);
	}

	ClockMode=SHOWSENSORS;
	nSensors = Search_sensors();
	digit=nSensors;
	display();
	_delay_ms(500);
	if (nSensors==0) beep();
	//if SELECT and SET pushed on startup, set the mode seconds are displayed.
	if (bit_is_clear(KPIN, KEYSELECT) && bit_is_clear(KPIN, KEYSET))	ClockMode=SETSECMODE;
	//if only SELECT pushed on startup, toggle CFUnit and USMode.
	if (bit_is_set(KPIN, KEYSELECT) && bit_is_clear(KPIN, KEYSET))	
	{
		CFUnit=!CFUnit; 
		USMode=!USMode; 
		eeprom_write_byte(&ECFUnit,CFUnit);
		eeprom_write_byte(&EUSMode,USMode);
		crc_write();
	}
	//if only SET pushed on startup, toggle Swing.
	if (bit_is_clear(KPIN, KEYSELECT) && bit_is_set(KPIN, KEYSET))	Swing=!Swing;	eeprom_write_byte(&ESwing,Swing); crc_write();

	display();				



}



//function to check if the alarm should start
void CheckAlarm(void)
{
	if (ALSet)
	{
	if ((ALHours==dt.hour) && (ALMinutes==dt.minute) && (dt.second==0))
		{
			AlarmOn=1;
			t3=0;
		}
	}
	if (AlarmOn)
	{
		beep();
		_delay_ms(100);
		t3++;
		if (t3==2900) AlarmOn=0; //2900=10 minutes
	}
}


//function which reads if there is any key pressed
uint8_t readkeys(void)
{
	uint16_t keypressed=0;

	//SELECT and SET keys simultaneously. Not used.
	if (bit_is_clear(KPIN, KEYSELECT) && bit_is_clear(KPIN, KEYSET))
	{
		keypressed=3;
		_delay_ms(200);
	}
	//SELECT key
	else
	if bit_is_clear(KPIN, KEYSELECT) 
	{
		keypressed=1;
		_delay_ms(200);
	}
	//SET key
	else
	if bit_is_clear(KPIN, KEYSET) 
	{
		keypressed=2;
		_delay_ms(200);
	}
	else	keypressed=0;
	return keypressed;

}

void ConvertCToF(void)
{
	if (CFUnit == 1) digit=digit*9/5+32;
}


void SetParams(uint8_t tpulsing)
{
	pulsing=tpulsing;
	dt=get_date_time();
	seconds=dt.second;
	_delay_ms(10);

}



//main loop
int main()
 
{
uint8_t key;
	init();
	_delay_ms(100);
//	beep();
	while(1)
	{
		//if any key pressed right after reset
		//entering SetSecMode 
		if ((Start==1) && (ClockMode==SETSECMODE))
		{
			key=0;
			t1=6000;
			Start=0;
		}
		else
			key=readkeys();
		//no key pressed
		if (key==0)
		{
			//void all long key presses
			switch (ClockMode)
			{
			case SHOWCLOCK:
				if (t2==0)
				{
					//show clock
					SetParams(0);
					//pulsing=0;
					//dt=get_date_time();
					//seconds=dt.second;
					digit=dt.hour*100+dt.minute;
				}
				else
				if (t2<=5)
				{
					//show the date
					SetParams(0);
					//pulsing=0;
	  				//dt=get_date_time();
					//seconds=dt.second;
					t1=6000;
					t2=0;
					ClockMode=SHOWDATE;
				}
				else
				if (t2<=10)
				{
					//set the alarm
					SetParams(1);
					//pulsing=1;
  					//dt=get_date_time();
					//seconds=dt.second;
					t1=900;
					t2=0;
					ClockMode=SETALMINUTES;
				}
				else
				{
					//set the date
					pulsing=1;
					t1=900;
					t2=0;
					dt1=dt;
					ClockMode=SETYEAR;
				}
				break;	
			case SHOWTEMP:	
				//show temperature until timer expires
				//pulsing=0;
  				//dt=get_date_time();
				//seconds=dt.second;
				//_delay_ms(10);
				switch (nSensors)
				{
					case 0:
						digit = 255;
					case 1:
						digit = ds18b20_gettemp();
						ConvertCToF();
					default:
						digit = ds18b20_getindextemp(&gSensorIDs[TEMPDISPLAY/2][0]);
						ConvertCToF();
				}
				SetParams(0);
				//digit = get_ds1621_temperature();
				break;	
			case SHOWDATE:
			//show the date
				SetParams(0);
				//pulsing=0;
				//dt=get_date_time();
				//seconds=dt.second;
				//_delay_ms(1);
				digit=dt.month*100+dt.date;
				break;	
			case SHOWYEAR:
			//show the year
				SetParams(0);
				//pulsing=0;
				//dt=get_date_time();
				//seconds=dt.second;
				//_delay_ms(1);
				digit=2000+dt.year;
				break;	
			case SETHOURS:
				SetParams(1);
				//pulsing=1;
  				//dt=get_date_time();
				//seconds=dt.second;
				//_delay_ms(10);
				break;	
			case SETMINUTES:	
				SetParams(1);
				//pulsing=1;
  				//dt=get_date_time();
				//seconds=dt.second;
				//_delay_ms(10);
				break;	
			case SETDATE:
				SetParams(1);
				//pulsing=1;
  				//dt=get_date_time();
				//seconds=dt.second;
				//_delay_ms(10);
				break;	
			case SETMONTH:
				SetParams(1);
				//pulsing=1;
  				//dt=get_date_time();
				//seconds=dt.second;
				//_delay_ms(10);
				break;	
			case SETYEAR:
				SetParams(1);
				//pulsing=1;
  				//dt=get_date_time();
				//seconds=dt.second;
				//_delay_ms(10);
				break;	
			case SETALHOURS:
				SetParams(1);
				//pulsing=1;
  				//dt=get_date_time();
				//seconds=dt.second;
				//_delay_ms(10);
				break;	
			case SETALMINUTES:
				SetParams(1);
				//pulsing=1;
  				//dt=get_date_time();
				//seconds=dt.second;
				//_delay_ms(10);
				break;	
			case SETAL:
				SetParams(1);
				//pulsing=1;
  				//dt=get_date_time();
				//seconds=dt.second;
				//_delay_ms(10);
				break;	
			case SETSECMODE:
				//read seconds to be displayed
				SetParams(1);
				//pulsing=1;
  				//dt=get_date_time();
				//seconds=dt.second;
				//_delay_ms(10);
				break;	
			default:
				break;	
			}
		}
		else
		//set key was pressed
		if (key==1)
		{
			if (AlarmOn)
			{
				AlarmOn=0;
			}
			switch (ClockMode)
			{
			case SHOWCLOCK:
				//set to SHOWTEMP and set the timer to ~3 seconds
				t1=90;
				ClockMode=SHOWTEMP;
				break;	
			case SHOWTEMP:
				t1=90;
				if (++TEMPDISPLAY/2==nSensors) TEMPDISPLAY=0;
				break;	
			case SHOWDATE:
				break;	
			case SHOWYEAR:
				break;	
			case SETHOURS:
				pulsing=0;
				t1=900;
				if (++dt1.hour>MaxHours) dt1.hour=MinHours;
				break;	
			case SETMINUTES:	
				pulsing=0;
				t1=900;
				if (++dt1.minute>MaxMinutes) dt1.minute=MinMinutes;
				break;	
			case SETDATE:
				pulsing=0;
				t1=900;
				if (++dt1.date>MaxDate) dt1.date=MinDate;
				break;	
			case SETMONTH:
				pulsing=0;
				t1=900;
				if (++dt1.month>MaxMonth) dt1.month=MinMonth;
				break;	
			case SETYEAR:
				pulsing=0;
				t1=900;
				if (++dt1.year>MaxYears) dt1.year=MinYears;
				break;	
			case SETALHOURS:
				pulsing=0;
				t1=900;
				if (++ALHours>MaxALHours) ALHours=MinALHours;
				break;	
			case SETALMINUTES:
				pulsing=0;
				t1=900;
				if (++ALMinutes>MaxALMinutes) ALMinutes=MinALMinutes; 
				break;	
			case SETAL:
				pulsing=0;
				t1=900;
				if (++ALSet>MaxALSet) ALSet=MinALSet;
				break;	
			case SETSECMODE:
				SetParams(0);
				//pulsing=0;
				//dt=get_date_time();
				//seconds=dt.second;
				t1=900;
				if (++SecMode>MaxSecMode) SecMode=MinSecMode;
				break;	
			default:
				break;	
			}
		}
		else
		//select key pressed
		if (key==2)
		{
			if (AlarmOn)
			{
				AlarmOn=0;
			}
			switch (ClockMode)
			{
			case SHOWCLOCK:
				//short key press 1-5 seconds: show date/time for 10 seconds
				//long key press 6-10 seconds: enter alarm set
				//more than 10 seconds pressed: enter date set
				t2++;
				digit=t2;
			case SHOWTEMP:
				break;	
			case SHOWDATE:
				t1=6000;
				ClockMode=SHOWYEAR;
				break;	
			case SHOWYEAR:
				ClockMode=SHOWCLOCK;
				break;	
			case SETHOURS:
				//write the time/date/year into the DS1302
				pulsing=0;
				set_date_time(dt1);
				t1=0;
				beep();
				_delay_ms(200);
				break;	
			case SETMINUTES:	
				ClockMode=SETHOURS;
				break;	
			case SETDATE:
				ClockMode=SETMINUTES;
				break;	
			case SETMONTH:
				ClockMode=SETDATE;
				break;	
			case SETYEAR:
				ClockMode=SETMONTH;
				break;	
			case SETALHOURS:
				ClockMode=SETAL;
				break;	
			case SETALMINUTES:
				ClockMode=SETALHOURS;
				break;	
			case SETAL:
				//write Alarm Values into EEPROM
				pulsing=0;
				eeprom_write_byte(&EALHours,ALHours);
				eeprom_write_byte(&EALMinutes,ALMinutes);
				eeprom_write_byte(&EALSet,ALSet);
				crc_write();
				t1=0;
				beep();
				_delay_ms(200);
				break;	
			case SETSECMODE:
				//write SecMode into EEPROM
				pulsing=0;
				eeprom_write_byte(&ESecMode,SecMode);
				crc_write();
				t1=0;
				beep();
				_delay_ms(200);
				break;	
			default:
				break;	
			}
		}
		t1--;
		if (t1==0xFFFF) t1=0;
		if (t1==0) {ClockMode=SHOWCLOCK; TEMPDISPLAY=0;}
		if ((Swing) && (ClockMode==SHOWCLOCK) && (seconds/10*10==seconds))
		{
					ClockMode=SHOWTEMP; 
					TEMPDISPLAY=1; 
		}
		display();
		CheckAlarm();
	}

}
 
